/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.beans.application;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.bean.ApplicationScoped;
import javax.faces.bean.ManagedBean;
import javax.faces.context.FacesContext;

import kieker.analysis.AnalysisController;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.common.ACManager;
import kieker.webgui.common.FSManager;
import kieker.webgui.common.exception.ProjectAlreadyExistingException;

/**
 * This bean contains all application wide available projects. It provides methods to create and manage the projects. Synchronization is achieved in the
 * {@link FSManager}.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@ManagedBean
@ApplicationScoped
public final class ProjectsBean {

	/**
	 * This is the log for errors, exceptions etc.
	 */
	private static final Log LOG = LogFactory.getLog(ProjectsBean.class);
	/**
	 * This list contains all available projects by name.
	 */
	private final List<String> projects = Collections.synchronizedList(new ArrayList<String>());

	/**
	 * Creates a new instance of this class.
	 */
	public ProjectsBean() {
		// No code necessary
	}

	/**
	 * Initializes this bean.
	 */
	@PostConstruct
	protected void init() {
		// Load a list with all available projects on the FS
		this.projects.addAll(FSManager.getInstance().getAllProjects());
	}

	/**
	 * Adds a new project to this application if possible. It informs the current user (via the growl-component of PrimeFaces) whether the creation was successful or
	 * not.
	 * 
	 * @param project
	 *            The name of the project to be added.
	 */
	public void addProject(final String project) {
		try {
			// Try and use the FS-Manager to create the project atomically.
			FSManager.getInstance().addProject(project);
			// If there were no exception, everything went well. We can add the project to our list.
			this.projects.add(project);
			// Inform the user
			ProjectsBean.showMessage(FacesMessage.SEVERITY_INFO, "Project created.");
		} catch (final IOException ex) {
			ProjectsBean.LOG.error("An error occured while creating the project.", ex);
			ProjectsBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while creating the project.");
		} catch (final ProjectAlreadyExistingException ex) {
			ProjectsBean.LOG.info("A project with the same name exists already.", ex);
			ProjectsBean.showMessage(FacesMessage.SEVERITY_WARN, "A project with the same name exists already.");
		}
	}

	/**
	 * This method can be used to open an existing project. This means that the current state of the project on the file system is loaded into an instance of {@link}
	 * MIProject}. This instance can be modified at will and for example later saved by the {@link FSManager}.
	 * 
	 * @param project
	 *            The name of the project to be saved.
	 * @return Either the model instance if the loading was successful or null otherwise. In the latter case, the user will be informed via the growl-component.
	 */
	public MIProject openProject(final String project) {
		try {
			return FSManager.getInstance().openProject(project);
		} catch (final IOException ex) {
			ProjectsBean.LOG.error("An error occured while loading the project.", ex);
			ProjectsBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while loading the project.");
			return null;
		}
	}

	/**
	 * This method renames the given project. For other threads active on this project it looks like the project has been removed in the meantime. If something goes
	 * wrong, the user will be informed via the growl-component.
	 * 
	 * @param projectName
	 *            The old name of the project.
	 * @param newName
	 *            The new name of the project.
	 */
	public void renameProject(final String projectName, final String newName) {
		try {
			FSManager.getInstance().renameProject(projectName, newName);
		} catch (final IOException ex) {
			ProjectsBean.LOG.error("An error occured while renaming the project.", ex);
			ProjectsBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while renaming the project.");
		} catch (final ProjectAlreadyExistingException ex) {
			ProjectsBean.LOG.info("A project with the same name exists already.", ex);
			ProjectsBean.showMessage(FacesMessage.SEVERITY_WARN, "A project with the same name exists already.");
		}
	}

	/**
	 * This method can be used to get the current time stamp of a given project as a human readable date. If the project doesn't exist or the time stamp would be
	 * invalid, the date of 0 is returned.
	 * 
	 * @param project
	 *            The project whose time stamp should be collected.
	 * @return The human readable time stamp of the project.
	 */
	public String getCurrTimeStamp(final String project) {
		// Get the current time stamp of the project...
		final long timeStamp = FSManager.getInstance().getCurrTimeStamp(project);
		// ...and make sure the user can read it.
		return new Date(timeStamp).toString();

	}

	/**
	 * This method delivers all available projects as a list of string.
	 * 
	 * @return All currently available projects. Do not modify this list with add, remove etc. (It still has to be sortable for Primefaces though)
	 */
	public List<String> getProjects() {
		return this.projects;
	}

	/**
	 * This method can be used to deliver the state of the analysis controller of the given project as a human readable string.
	 * 
	 * @param project
	 *            The project whose state should be delivered.
	 * @return The current state of the corresponding {@link AnalysisController} as defined by {@link ACManager#getAnalysisControllerStateString(String)}.
	 */
	public String getAnalysisControllerState(final String project) {
		return ACManager.getInstance().getAnalysisControllerStateString(project);
	}

	/**
	 * This method shows the current user a message by using the growl-component of PrimeFaces.
	 * 
	 * @param severity
	 *            The severity of the message.
	 * @param msg
	 *            The message itself.
	 */
	private static void showMessage(final Severity severity, final String msg) {
		FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(severity, "", msg));
	}
}
